class TweetParser(object):
    def __init__(self):
        pass
    '''
        get an entity from a tweet if it exists
    '''
    def get_entity(self, entity_type, tweet):
        if self.contains_entity(entity_type, tweet):
            return [entity for entity in tweet["entities"][entity_type]]
        return []

    '''
        returns True if tweet contains one or more entities (hashtag, url, or media)
    '''
    @staticmethod
    def contains_entity(entity_type, tweet):
        if "entities" not in tweet:
            return False
        elif entity_type in tweet["entities"] and len(tweet["entities"][entity_type]) > 0:
            return True
        return False

    '''
        gets a particular field for an entity if it exists
    '''
    @staticmethod
    def get_entity_field(field, entity):
        # beacuse all entities are actually lists
        # of entity objects
        for entity_object in entity:
            if field in entity_object:
                return entity[field]
        return None

    '''
        tests a tweet to see if it passes a 
        custom filter method, this just returns the
        value of the filter method passed in
    '''
    @staticmethod
    def tweet_passes_custom_filter(function, tweet):
        return function(tweet)

    '''
        removes all the the specified field from a tweet
    '''
    @staticmethod
    def strip_tweet(keep_fields, tweet):
        stripped_tweet = {}
        expanded_fields = [field_path.split('.') for field_path in keep_fields]
        for expanded_field in expanded_fields:
            prev = {}
            prev_tweet = {}
            temp_iteration_dict = {}
            for count, field in enumerate(expanded_field):
                #if its a top level field
                if field in tweet:
                    if count+1 == len(expanded_field):
                        temp_iteration_dict[field] = tweet[field]
                    else: 
                        temp_iteration_dict[field] = {}
                    prev_tweet = tweet[field]
                    prev = temp_iteration_dict[field]
                # if its a mid level field
                elif field in prev_tweet:
                    if count+1 == len(expanded_field):
                        prev[field] = prev_tweet[field]
                    else: 
                        prev[field] = {}
                    prev_tweet = prev_tweet[field]
                    prev = prev[field]
            # merge into main dict
            c = temp_iteration_dict.copy()
            stripped_tweet.update(c)
        return stripped_tweet

    '''
        just tests multiple custom filters
        see, tweet_passes_custom_filter
    '''
    def tweet_passes_custom_filter_list(self, function_list, tweet):
        for function in function_list:
            if not self.tweet_passes_custom_filter(function, tweet):
                return False
        return True

    '''
        return true or false depends if
        tweet passes through the filter
        filters are just dictionaries.
        filter = mongo style query dict
    '''
    def tweet_passes_filter(self, filter_obj, tweet):
        if filter_obj == {}:
            return True
        # lists of tuples that
        # come from our dicts
        flat_tweet_list = []
        for tweet_tuple in self.flatten_dict(tweet):
            flat_tweet_list.append(tweet_tuple)
        for filter_tuple in self.flatten_dict(filter_obj):
            if filter_tuple not in flat_tweet_list:
                return False
        return True

    '''
        get a list where each element in the list
        is a tuple that contains, (['path','to','value'], value_at_path)
    '''
    def flatten_dict(self, dict_obj, path=None):
        if path is None:
            path = []
        if isinstance(dict_obj, dict):
            for key in dict_obj.keys():
                local_path = path[:]
                local_path.append(key)
                for val in self.flatten_dict(dict_obj[key], local_path):
                    yield val
        else:
            yield path, dict_obj

    '''
        pulls out the columns from a tweet, usually used for making
        tweets into a fixed schema/csv/sql table/columnd based data
        structure, used for csv dump and sqlite db generation
    '''
    def parse_columns_from_tweet(self, tweet, columns):
        def return_val_for_column(tweet, columns):
            temp_tweet = {}
            for sub_field in columns:
                if temp_tweet == {}:
                    temp_tweet = tweet
                try:
                    if sub_field.isdigit():
                        sub_field = int(sub_field)
                    val = temp_tweet[sub_field]
                    if isinstance(val,dict) or isinstance(val,list):
                        temp_tweet = val
                        continue
                    else: 
                        if isinstance(val,str):
                            val = val.replace('\n',' ').replace('\r',' ')
                        return val
                except (KeyError, IndexError) as e:
                    return None
                    break
        ret_columns = []
        for field in columns:
            split_field = field.split('.')
            ret_columns.append((field,return_val_for_column(tweet, split_field)))
        return ret_columns

'''
author @yvan
tweet parser is a tool for making tweet filters
'''


import os
import unittest

from test.config import config
from smappdragon import BsonCollection
from smappdragon.tools.tweet_parser import TweetParser

class TestBsonCollection(unittest.TestCase):

	def test_iterator_returns_tweets(self):
		collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
		self.assertTrue(len(list(collection.get_iterator())) > 0)

	# special test because custom logic is different on mongo
	def test_bson_collection_custom_filter_filters(self):
		collectionone = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
		full_collection_len = len(list(collectionone.get_iterator()))
		def is_tweet_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return True
			else:
				return False
		num_retweets = len(list(collectionone.set_custom_filter(is_tweet_a_retweet).get_iterator()))

		collectiontwo = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
		def is_not_a_retweet(tweet):
			if 'retweeted' in tweet and tweet['retweeted']:
				return False
			else:
				return True
		num_non_retweets = len(list(collectiontwo.set_custom_filter(is_not_a_retweet).get_iterator()))
		
		#the numbes of retweets and non retweets should add up to the whole collection
		self.assertEqual(num_retweets + num_non_retweets, full_collection_len)
		
	def test_strip_tweets_keeps_fields(self):
		tweet_parser = TweetParser()
		collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
		self.maxDiff = None
		it = collection.strip_tweets(['id', 'entities.user_mentions', 'user.profile_image_url_https']).get_iterator()
		def tweets_have_right_keys(iterator, fields):
			for tweet in iterator:
				keys = [key for key,value in tweet_parser.flatten_dict(tweet)]
				for elem in fields:
					if elem not in keys:
						return False
			return True		
		self.assertTrue(tweets_have_right_keys(it, [['id'], ['entities', 'user_mentions'], ['user', 'profile_image_url_https']]))

if __name__ == '__main__':
	unittest.main()

'''
read about twitter entities here:
https://dev.twitter.com/overview/api/entities-in-twitter-objects
'''


'''
module indicator for collection
'''
from . import mongo_collection
from . import bson_collection
from . import json_collection
from . import csv_collection
__all__ = ['mongo_collection', 'bson_collection', 'json_collection', 'csv_collection']



import os
import bson
import csv
import json

def clean_tweets(input_type, input_file_path, output_file_path, error_file_path):
    if input_type == 'bson':
        bson_handle = open(input_file_path, 'rb')
        try:
            for count, line in enumerate(bson.decode_file_iter(bson_handle)):
                pass
        except:
            pass
    elif input_type == 'json':
        json_handle = open(input_file_path, 'r', encoding='utf-8')
        with open(output_file_path, 'w', encoding='utf-8') as fo:
            with open(error_file_path, 'w', encoding='utf-8') as f:
                for count, line in enumerate(json_handle):
                    try:
                        tweet = json.loads(line)
                        fo.write(json.dumps(tweet))
                        fo.write('\n')
                    except:
                        f.write(line)
        json_handle.close()

class SmappError(Exception):
    pass


'''
@yvan
can be used to clean tweets in a general catch all sense
kept separate from the data sources bson_collection, etc
to keep datasource implementation simple, its also not
a core function, its really more of an extra, how you clean
your data is up to you, we jsut offer this way. 
methods can get big and out of hand very quickly for cleaning
data. so im putting this here in an attempt to keep other parts
of the code from getting too crazy and unmaintainable.
'''


import os
import json
import pymongo
import unittest

from test.config import config
from smappdragon import JsonCollection
from smappdragon.tools.tweet_cleaner import clean_tweets

class TestTweetCleaner(unittest.TestCase):

    def setUp(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json')

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json')
            
    def tearDown(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json')

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json')

    def test_clean_tweets_on_dirty_data(self):
        self.setUp()
        clean_tweets('json', 
        os.path.dirname(os.path.realpath(__file__)) +'/'+ config['json']['dirty'], 
        os.path.dirname(os.path.abspath(__file__))+'/../test/output.json', 
        os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json')

        col = JsonCollection(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json')

        with open(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json', 'r') as f:
            for line in f:
                try:
                    json.loads(line)
                except:
                    self.assertTrue(False)

        excepted = False
        with open(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json', 'r') as f:
            for line in f:
                try:
                    json.loads(line)
                except:
                    excepted = True
        self.assertTrue(excepted)
        self.tearDown()

    def test_clean_tweets_on_clean_data(self):
        self.setUp()
        clean_tweets('json', 
        os.path.dirname(os.path.realpath(__file__)) +'/'+ config['json']['valid'], 
        os.path.dirname(os.path.abspath(__file__))+'/../test/output.json', 
        os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json')

        col = JsonCollection(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json')

        with open(os.path.dirname(os.path.abspath(__file__))+'/../test/output.json', 'r') as f:
            for line in f:
                try:
                    json.loads(line)
                except:
                    self.assertTrue(False)

        excepted = False
        with open(os.path.dirname(os.path.abspath(__file__))+'/../test/output_err.json', 'r') as f:
            for line in f:
                try:
                    json.loads(line)
                except:
                    print('blah')
                    excepted = True
        self.assertFalse(excepted)
        self.tearDown()            

if __name__ == '__main__':
    unittest.main()

'''
author @yvan
'''


import pymongo

from smappdragon.tools.tweet_parser import TweetParser
from smappdragon.collection.base_collection import BaseCollection

class MongoCollection(BaseCollection):
	'''
		method that tells us how to
		create the MongoCollection object
		*args -> address, port, username, password, database_name, collection_name
	'''
	def __init__(self, *args, **kwargs):
		BaseCollection.__init__(self)
		if 'passed_mongo' in kwargs:
			self.mongo = kwargs['passed_mongo']
			self.mongo_database = self.mongo[args[2]]
			if args[0] and args[1]:
				self.mongo_database.authenticate(args[0], args[1])
			self.mongo_collection = self.mongo_database[args[3]]
		else:
			self.mongo = pymongo.MongoClient(args[0], int(args[1]))
			self.mongo_database = self.mongo[args[4]]
			if args[2] and args[3]:
				self.mongo_database.authenticate(args[2], args[3])
			self.mongo_collection = self.mongo_database[args[5]]

	'''
		method that creates a cursor
		and yields all tweets in a particular collection
	'''
	def get_iterator(self):
		tweet_parser = TweetParser()
		mongo_cursor = self.mongo_collection.find( \
			filter=self.filter, \
			no_cursor_timeout=False, \
			limit=self.limit \
		)
		for tweet in mongo_cursor:
			if tweet_parser.tweet_passes_custom_filter_list(self.custom_filters, tweet):
				if self.should_strip:
					yield tweet_parser.strip_tweet(self.keep_fields, tweet) 
				else: 
					yield tweet
		mongo_cursor.close()



"""
Tweet filter module,
contains common shared tweet filter functions

Functions and docstrings parsed directly by the web dashboard for adding these to tweet collector.

2014/11/19 @jonathanronen
"""

def field_contains(tweet, field, *terms, **kwargs):
    """
    Returns true if the text in tweet[field] contains any of the terms given.
    By default, this function is NOT case-sensitive.

    `field` may be any attribute of tweets, for instance:
        field_contains(tweet, 'text', 'nyc')
        # will return True for tweets containing nyc

    `field` may also be a path to a nested attribute, for instance:
        field_contains(tweet, 'user.screen_name', 'bob', 'alice')
        # will return True for usernames with bob or alice in them.

    Example:
    ========
    field_contains(tweet, 'user.screen_name', 'obama', 'putin')
    # true if the user's handle contains 'obama' or 'putin'
    """
    path = field.split('.')
    value = tweet
    for p in path:
        value = value[p]
    if kwargs.get("case_sensitive", False):
        return any(term in value for term in terms)
    else:
        value = value.lower()
        return any(term.lower() in value for term in terms)

def field_contains_case_sensitive(tweet, field, *terms):
    """
    Returns true if the text in tweet[field] contains any of the terms given.
    Terms are case-sensitive.

    `field` may be any attribute of tweets, for instance:
        field_contains(tweet, 'text', 'NYC')
        # will return True for tweets containing NYC 
        # will return false if tweet contains only lowercase nyc

    `field` may also be a path to a nested attribute, for instance:
        field_contains(tweet, 'user.screen_name', 'Bob', 'ALICE')
        # will return True for usernames with Bob or ALICE in them.

    Example:
    ========
    field_contains_case_sensitive(tweet, 'text', ICE', 'IRA')
    # true if tweet contains ICE or IRA (but false for lowercase ice, ira)
    """
    if field_contains(tweet, field, *terms, case_sensitive=True):
        return True
    elif field_contains(tweet, field, *terms, case_sensitive=False):
        return False
    return True

def user_location_contains(tweet, *terms):
    """
    True if tweet['user']['location'] contains any of the terms.
    """
    return field_contains(tweet, 'user.location', *terms)

def user_description_contains(tweet, *terms):
    """
    True if tweet['user']['description'] contains any of the terms.
    """
    return field_contains(tweet, 'user.description', *terms)

def within_geobox(tweet, sw_lon, sw_lat, ne_lon, ne_lat):
    """
    True if tweet is geotagged and is sent within the box specified by GeoJSON points (longitude, latitude)
    (sw_lon, sw_lat)  <-  the southwest corner
    (ne_lon, ne_lat)  <-  the northeast corner

    Example:
    ========
    within_geobox(tweet, -75.280303,39.8670041,-74.9557629,40.1379919)
    # true for tweets tweeted within a box surrounding Philadelphia
    """
    if 'coordinates' not in tweet or tweet['coordinates'] is None or 'coordinates' not in tweet['coordinates']:
        return False
    coords = tweet['coordinates']['coordinates']
    return coords[0] > float(sw_lon) and coords[0] < float(ne_lon) and coords[1] > float(sw_lat) and coords[1] < float(ne_lat)

def place_name_contains(tweet, *terms):
    """
    True if the `place` associated with the tweet contains any of the terms
    For more information about `place see https://dev.twitter.com/overview/api/places

    Example:
    ========
    place_name_contains(tweet, 'Kiev')
    # true for tweets where tweet['place']['full_name'] contains 'kiev'.
    """
    if 'place' not in tweet or tweet['place'] is None:
        return False
    return field_contains(tweet, 'place.full_name', *terms) or field_contains(tweet, 'place.country', *terms)



import os
import csv
import sqlite3
import unittest

from bson import json_util
from test.config import config
from smappdragon import BsonCollection
from smappdragon import MongoCollection
from smappdragon import JsonCollection

class TestBaseCollection(unittest.TestCase):

    def test_limit_is_set(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.set_limit(5)
        self.assertEqual(5, collection.limit)
        collection.set_limit(0)

    def test_limit_actually_limits(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.get_iterator()
        count = len(list(tweet for tweet in collection.set_limit(5).get_iterator()))
        self.assertEqual(5, count)

    def test_filter_is_set(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.set_filter({'a':'b', 'c':'d', 'e':{'f':'g', 'h':'i'}})
        self.assertEqual(collection.filter, {'a':'b', 'c':'d', 'e':{'f':'g', 'h':'i'}})

    def test_dump_to_bson_dumps(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson')

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.bson'
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.dump_to_bson(output_path)
        self.assertTrue(os.path.getsize(output_path) > 0)

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson')

    def test_dump_to_json_dumps(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson.json')

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.bson.json'
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.dump_to_json(output_path)
        self.assertTrue(os.path.getsize(output_path) > 0)

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson.json'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.bson.json')

    def test_dump_to_csv_orders_and_encodes_properly(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv')

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.csv'
        collection = JsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['json']['valid-single'])
        collection.dump_to_csv(output_path, ['id_str', 'entities.hashtags.0.text', 'entities.hashtags.1.text', 'source', 'user.id', 'timestamp.$date', 'text'])
        with open(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv', 'r') as filehandle:
            count = 0
            for line in csv.reader(filehandle):
                if count != 0:
                    val_count = 0
                    for csv_row_value in line:
                        everything_in_order = True
                        if val_count == 0:
                            self.assertEqual(csv_row_value, '661275583813431296')
                        elif val_count == 1:
                            if csv_row_value != 'jadehelm':
                                everything_in_order = False
                        elif val_count == 2:
                            if csv_row_value != 'newworldorder':
                                everything_in_order = False
                        elif val_count == 3:
                            self.assertEqual(csv_row_value, '<a href="https://twitter.com/Col_Connaughton" rel="nofollow">Colin\'s Autotweeterpro5.3</a>')
                        elif val_count == 4:
                            self.assertEqual(csv_row_value, '379851447')
                        elif val_count == 5:
                            self.assertEqual(csv_row_value, '2015-11-02 20:15:59+00:00')
                        elif val_count == 6:
                            self.assertEqual(csv_row_value, 'Susan Lindauer, Rtd US Army LTC Potter: Jade Helm https://t.co/VA4bQRudLt #jadehelm #newworldorder #usa #tyranny #threat')
                        if everything_in_order:
                            self.assertTrue(True)
                        val_count += 1
                else:
                    count += 1
        filehandle.close()
        
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv')

    def test_dump_to_csv_dumps(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv')

        field_list = ['id_str',
        'coordinates.coordinates.0',
        'coordinates.coordinates.1',
        'user.id_str',
        'user.lang',
        'lang',
        'text',
        'user.screen_name',
        'user.location',
        'user.description',
        'created_at',
        'user.friends_count',
        'user.followers_count',
        'retweet_count',
        'entities.urls.0.expanded_url',
        'entities.urls.1.expanded_url',
        'entities.urls.2.expanded_url',
        'entities.urls.3.expanded_url',
        'entities.urls.4.expanded_url',
        'entities.hashtags.0.text',
        'entities.hashtags.1.text']

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.csv'
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.dump_to_csv(output_path, field_list)
        self.assertTrue(os.path.getsize(output_path) > 0)

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.csv')

    def test_dump_to_sqlite_db_dumps(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.db'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.db')

        field_list = ['id_str',
        'coordinates.coordinates.0',
        'coordinates.coordinates.1',
        'user.id_str',
        'user.lang',
        'lang',
        'text',
        'user.screen_name',
        'user.location',
        'user.description',
        'created_at',
        'user.friends_count',
        'user.followers_count',
        'retweet_count',
        'entities.urls.0.expanded_url',
        'entities.urls.1.expanded_url',
        'entities.urls.2.expanded_url',
        'entities.urls.3.expanded_url',
        'entities.urls.4.expanded_url',
        'entities.hashtags.0.text',
        'entities.hashtags.1.text']

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.db'
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.dump_to_sqlite_db(output_path, field_list)

        self.assertTrue(os.path.getsize(output_path) > 0)

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.db'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.db')

    def test_dump_to_sqlite_db_dumps_the_right_stuff(self):
        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.db'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.db')

        field_list = ['id_str',
        'coordinates.coordinates.0',
        'coordinates.coordinates.1',
        'user.id_str',
        'user.lang',
        'lang',
        'text',
        'user.screen_name',
        'user.location',
        'user.description',
        'created_at',
        'user.friends_count',
        'user.followers_count',
        'retweet_count',
        'entities.urls.0.expanded_url',
        'entities.urls.1.expanded_url',
        'entities.urls.2.expanded_url',
        'entities.urls.3.expanded_url',
        'entities.urls.4.expanded_url',
        'entities.hashtags.0.text',
        'entities.hashtags.1.text']

        output_path = os.path.dirname(os.path.realpath(__file__)) + '/' + 'data/output.db'
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        collection.dump_to_sqlite_db(output_path, field_list)

        con = sqlite3.connect(output_path)
        cur = con.cursor()
        row = [elem for row in cur.execute("SELECT * FROM data LIMIT 1;") for elem in row ]
        con.close()
        self.assertTrue(len(row) > 0)
        self.assertEqual(set(row), set(['661275583813431296', 'NULL', 'NULL', '379851447', 'en', 'de', 'Susan Lindauer, Rtd US Army LTC Potter: Jade Helm https://t.co/VA4bQRudLt #jadehelm #newworldorder #usa #tyranny #threat', 'Col_Connaughton', 'London UK', '#gaza #palestine #israel #BDS MAD EVIL ISRAEL MURDERS BABIES CIVILIANS to STEAL PALESTINIAN LAND RESOURCES with USA UK HELP. To stop my tweets, BLOCK or MUTE me', 'Mon Nov 02 20:15:59 +0000 2015', 2019, 3159, 0, 'https://www.youtube.com/watch?v=0nJqymxVpwc', 'NULL', 'NULL', 'NULL', 'NULL', 'jadehelm', 'newworldorder']))

        if os.path.exists(os.path.dirname(os.path.abspath(__file__))+'/data/output.db'):
            os.remove(os.path.dirname(os.path.abspath(__file__))+'/data/output.db')

    def test_set_custom_filter_is_set(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        def is_tweet_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return True
            else:
                return False
        collection.set_custom_filter(is_tweet_a_retweet)
        self.assertTrue(len(collection.custom_filters) == 1)

    def test_set_custom_filter_is_not_double_set(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        def is_tweet_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return True
            else:
                return False
        collection.set_custom_filter(is_tweet_a_retweet)
        self.assertFalse(len(collection.custom_filters) > 1)

    def test_collection_filters_custom_filter_filters_something(self):
        collection = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        long_len = len(list(collection.get_iterator()))
        def is_tweet_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return True
            else:
                return False
        collection.set_custom_filter(is_tweet_a_retweet)
        shorter_len = len(list(collection.get_iterator()))

        #there should be fewer retweets than all tweets.
        self.assertTrue(long_len > shorter_len)

    def test_collection_filters_custom_filter_properly_applies_filter(self):
        collectionone = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        full_collection_len = len(list(collectionone.get_iterator()))
        def is_tweet_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return True
            else:
                return False
        num_retweets = len(list(collectionone.set_custom_filter(is_tweet_a_retweet).get_iterator()))


        collectiontwo = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        def is_not_a_retweet(tweet):
            if 'retweeted' in tweet and tweet['retweeted']:
                return False
            else:
                return True
        num_non_retweets = len(list(collectiontwo.set_custom_filter(is_not_a_retweet).get_iterator()))

        #the numbes of retweets and non retweets should add up to the whole collection
        self.assertEqual(num_retweets + num_non_retweets, full_collection_len)

    def test_strip_tweets_strips_many_tweets_totally(self):
        collectionone = BsonCollection(os.path.dirname(os.path.realpath(__file__)) +'/'+ config['bson']['valid'])
        iterator = collectionone.strip_tweets([]).get_iterator()
        first_tweet = next(iterator)
        second_tweet = next(iterator)
        #exhaust the iterator
        len(list(iterator))
        self.assertTrue(first_tweet == {} and second_tweet == {})

if __name__ == '__main__':
    unittest.main()



'''
module marker for smappdragon
'''

from smappdragon.collection.bson_collection import BsonCollection
from smappdragon.collection.json_collection import JsonCollection
from smappdragon.collection.mongo_collection import MongoCollection
from smappdragon.collection.csv_collection import CsvCollection
from smappdragon.tools.tweet_parser import TweetParser
